/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const CLASS_ID = Components.ID("4bc9c264-b6c0-11dc-8314-0800200c9a66");
const CLASS_NAME = "Download Manager Service";
const CONTRACT_ID = "@thumbsdown.mozdev.org/download-manager-service;1";
const CATEGORY_ENTRY = "thumbsdown";

const Cc = Components.classes;
const Ci = Components.interfaces;

const STRING_BUNDLE_URI = "chrome://thumbsdown/locale/downloadmanagerservice.properties";


function DownloadManagerService()
{
    this.wrappedJSObject = this;
    
    this.dms = {"DownThemAll!": DownThemAll};
    
    var loader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
    loader.loadSubScript("chrome://global/content/nsUserSettings.js");
    loader.loadSubScript("chrome://global/content/strres.js");
    loader.loadSubScript("chrome://thumbsdown/content/util.js");
    
    // localization (util.js)
    stringBundle = srGetStrBundle(STRING_BUNDLE_URI);
    
    // check that dm is installed, is valid version and is not disabled
    var extMgr = Cc["@mozilla.org/extensions/manager;1"].getService(Ci.nsIExtensionManager);
    var extMgrDs = extMgr.datasource;
    var compareVersions = Cc["@mozilla.org/xpcom/version-comparator;1"].getService(Ci.nsIVersionComparator).compare;
    var rdfSvc = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
    for each (var dm in this.dms)
    {
        dm.enabled = false;
        var ext = extMgr.getItemForID(dm.id);
        if (ext)
        {
            dm.version = ext.version;
            if ((compareVersions(dm.version, dm.minVersion) >= 0) &&
                (compareVersions(dm.version, dm.maxVersion) <= 0))
            {
                dm.versionMatch = true;
                
                var source = rdfSvc.GetResource("urn:mozilla:item:" + dm.id);
                var property = rdfSvc.GetResource("http://www.mozilla.org/2004/em-rdf#isDisabled");
                var target = rdfSvc.GetLiteral("true");
                var disabled = extMgrDs.HasAssertion(source, property, target, true);
                dm.enabled = (! disabled);
            }
        }
    }
}

DownloadManagerService.prototype =
{
    // nsISupports
    QueryInterface: function(iid)
    {
        if (iid.equals(Ci.nsISupports) ||
            iid.equals(Ci.nsISupportsWeakReference))
            return this;
        throw Cr.NS_ERROR_NO_INTERFACE;
    },
    
    download: function(urls, dir)
    {
        if (! (urls && urls.length > 0))
            return;
        // dta hard coded because it's the only dm at the moment
        var name = "DownThemAll!";
        var dm = this.dms[name];
        if (! dm)
            throw new Error(_("error.unknown", name));
        if (! dm.enabled)
        {
            if (! dm.version)
                throw new Error(_("error.notInstalled", name));
            if (! dm.versionMatch)
                throw new Error(_("error.incompatibleVersion", name, dm.version, dm.minVersion, dm.maxVersion));
            if (! dm.enabled)
                throw new Error(_("error.disabled", name));
        }
        dm.download(urls, dir);
        return true;
    }
}


var DownThemAll =
{
    id: "{DDC359D1-844A-42a7-9AA1-88A850A938A8}",
    minVersion: "1.0",
    maxVersion: "1.1.*",
    
    windowUri: "chrome://dta/content/dta/manager.xul",
    windowName: "",
    windowFeatures: "chrome, centerscreen, resizable=yes, dialog=no, all, modal=no, dependent=no",

    get numInstance()
    {
        // downthemall/chrome/content/common/overlayFunctions.js:DTA_AddingFunctions.turboSendToDown()
        var num = nsPreferences.getIntPref("extensions.dta.counter", 0);
        num = ((num < 999) ? ++num : 1);
        nsPreferences.setIntPref("extensions.dta.counter", num);
        return num;
    },
    
    get renameMask() { return nsPreferences.copyUnicharPref("extensions.thumbsdown.dm.dta.renameMask"); },
    
    download: function(urls, dir)
    {
        var queue = true;
        var numInstance = this.numInstance;
        var renameMask = this.renameMask;
        // downthemall/chrome/content/common/overlayFunctions.js:DTA_AddingFunctions.saveSingleLink()
        // downthemall/chrome/content/common/overlayFunctions.js:DTA_AddingFunctions.turboSendToDown()
        var dtaUrls = [{url: url.url, dirSave: dir, numIstance: numInstance, mask: renameMask, referrer: url.referer} for each (url in urls)];
        this._start(queue, dtaUrls)
    },
    
    _start: function(queue, dtaUrls)
    {
        // downthemall/chrome/content/common/overlayFunctions.js:DTA_AddingFunctions.sendToDown()
        var win = this._getWindow(this.windowUri);
        // if window is open...
        if (win)
            // ...and initialized, use it
            if (win.self && win.self.Dialog && win.self.Dialog._initialized)
                win.self.startDownloads(queue, dtaUrls);
            else
            {
                var onLoad = function(e)
                {
                    // |this| is dta manager window
                    var startDownloads = function() { this.startDownloads(queue, dtaUrls); }
                    // 0-delay required to make sure that dta onload handler goes first
                    e.target.defaultView.setTimeout(startDownloads, 0);
                }
                win.addEventListener("load", onLoad, false);
            }
        // if window is not yet opened, open a new one
        else
        {
            var windowWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(Ci.nsIWindowWatcher);
            var parent = null;
            var arguments = null;
            win = windowWatcher.openWindow(parent, this.windowUri, this.windowName, this.windowFeatures, arguments);
            // FIXME: openWindow arguments hack
            win.arguments = [queue, dtaUrls];
        }
    },
    
    _getWindow: function(uri)
    {
        // downthemall/chrome/content/common/overlayFunctions.js:DTA_Mediator.get()
        var windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
        // in dta 1.0 there's no windowtype but in 1.1 it has been added ("DTA:Manager")
        var enumerator = windowMediator.getEnumerator(null);
        while (enumerator.hasMoreElements())
        {
            var win = enumerator.getNext();
            if (win.location == uri)
                return win;
        }
        return null;
    }
}


// nsIFactory
var DownloadManagerServiceFactory =
{
    instance: null,
    
    createInstance: function(outer, iid)
    {
        if (outer != null)
            throw Cr.NS_ERROR_NO_AGGREGATION;
        if (this.instance == null)
            this.instance = new DownloadManagerService();
        return this.instance.QueryInterface(iid);
    }
};

// nsIModule
var DownloadManagerServiceModule =
{
    registerSelf: function(compMgr, fileSpec, location, type)
    {
        // register
        compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, fileSpec, location, type);
        // add start-up observer
        //var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
        //catMgr.addCategoryEntry("xpcom-startup", CATEGORY_ENTRY, CONTRACT_ID, true, true);
    },
    
    unregisterSelf: function(compMgr, fileSpec, location)
    {
        // remove start-up observer
        //var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
        //catMgr.deleteCategoryEntry("xpcom-startup", CATEGORY_ENTRY, true);
        // unregister
        compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(CLASS_ID, location);
    },
    
    getClassObject: function(compMgr, cid, iid)
    {
        if (! cid.equals(CLASS_ID))
            throw Cr.NS_ERROR_NO_INTERFACE;
        if (! iid.equals(Ci.nsIFactory))
            throw Cr.NS_ERROR_NOT_IMPLEMENTED;
        return DownloadManagerServiceFactory;
    },
    
    canUnload: function(compMgr)
    {
        return true;
    }
};

function NSGetModule(compMgr, fileSpec)
{
    return DownloadManagerServiceModule;
}
