/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const Cc = Components.classes;
const Ci = Components.interfaces;

var promptSvc = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
var scriptMgr = Cc["@thumbsdown.mozdev.org/script-manager;1"].getService(Ci.nsISupports).wrappedJSObject;

var scriptTree;
var scriptTreeView;

function onLoad()
{
    stringBundle = $("stringbundle");
    
    $("pref_openScriptRunner").addEventListener("change", onOpenScriptRunnerChange, false);
    onOpenScriptRunnerChange();
    
    scriptTree = $("tree_scripts");
    scriptTreeView = new ScriptTreeView(scriptMgr.scripts);
    scriptTree.view = scriptTreeView;
    
    _updateScriptPane();
    
    var args = (window.arguments ? window.arguments[0] : {});
    this.showPane(args.pane);
}

function showPane(prefPaneName)
{
    if (prefPaneName)
        $("thumbsdown_prefwindow").showPane($("prefpane_" + prefPaneName));
}

function pickDownloadDir()
{
    var pref = $("pref_downloadDir");
    
    try
    {
        var dir = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
        dir.initWithPath(pref.value);
    }
    catch (e)
    {
        dir = null;
    }
    
    if (! (dir && dir.exists() && dir.isDirectory()))
        dir = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("Desk", Ci.nsIFile);
    
    var fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
    fp.init(window, _("folderPickerTitle"), Ci.nsIFilePicker.modeGetFolder);
    fp.displayDirectory = dir;
    var value = fp.show();
    if (value == Ci.nsIFilePicker.returnOK)
        pref.value = fp.file.path;
}

function onOpenScriptRunnerChange()
{
    $("pref_focusScriptRunner").disabled = (! $("pref_openScriptRunner").value);
}

function onScriptSelect()
{
    _updateScriptPane();
}

function addScript()
{
    var script = _openScriptDialog();
    if (! script)
        return;
    var id = scriptMgr.addScript(script);
    
    var scripts = scriptMgr.scripts;
    var rowIndex = [script.id for each (script in scripts)].indexOf(id);
    scriptTreeView.model = scripts;
    scriptTreeView.treeBoxObject.rowCountChanged(rowIndex, 1);
    
    _updateScriptTreeSelection(rowIndex);
}

function editScript()
{
    var rowIndex = scriptTree.currentIndex;
    scriptTreeView.treeBoxObject.ensureRowIsVisible(rowIndex);
    var oldScript = scriptTreeView.model[rowIndex];
    var script = _openScriptDialog(oldScript);
    if (! script)
        return;
    script.id = oldScript.id;
    scriptMgr.setScript(script);
    
    var scripts = scriptMgr.scripts;
    rowIndex = [script.id for each (script in scripts)].indexOf(script.id);
    scriptTreeView.model = scripts;
    scriptTreeView.treeBoxObject.invalidate();
    
    _updateScriptTreeSelection(rowIndex);
}

function removeScript()
{
    var rowIndex = scriptTree.currentIndex;
    scriptTreeView.treeBoxObject.ensureRowIsVisible(rowIndex);
    var script = scriptTreeView.model[rowIndex];
    var result = promptSvc.confirm(null, _("remove.confirm.title"), _("remove.confirm.msg", script.name));
    if (! result)
        return;
    scriptMgr.removeScript(script.id);
    
    var scripts = scriptMgr.scripts;
    scriptTreeView.model = scripts;
    scriptTreeView.treeBoxObject.rowCountChanged(rowIndex, -1);
    
    _updateScriptTreeSelection(rowIndex);
}

function _openScriptDialog(script)
{
    var params = {arg: script, result: null};
    document.documentElement.openSubDialog("chrome://thumbsdown/content/script.xul", null, params);
    return params.result;
}

function _updateScriptTreeSelection(rowIndex)
{
    if (rowIndex >= scriptTreeView.rowCount)
        rowIndex--;
        
    callLater(0, null, function()
    {
        scriptTreeView.selection.select(rowIndex);
        scriptTreeView.treeBoxObject.ensureRowIsVisible(rowIndex);
        _updateScriptPane();
    });
}

function _updateScriptPane()
{
    $("cmd_editScript").setAttribute("disabled", (scriptTree.currentIndex == -1));
    $("cmd_removeScript").setAttribute("disabled", (scriptTree.currentIndex == -1));
}


function ScriptTreeView(scripts) // extend TreeView
{
    TreeView.call(this, scripts);
}

ScriptTreeView.prototype =
{
    getCellText: function(row, col)
    {
        var script = this.model[row];
        switch (col.id)
        {
            case "tcol_script_name":
                return script.name;
            
            case "tcol_script_type":
                return _("type." + script.type);
        }
        throw new Error("unknown column: " + col.id);
    }
}

extend(ScriptTreeView, TreeView);
